"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _moment = _interopRequireDefault(require("moment"));

var _fileStreamRotator = _interopRequireDefault(require("file-stream-rotator"));

var _zlib = _interopRequireDefault(require("zlib"));

var _fs = _interopRequireDefault(require("fs"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var FileLogger = /*#__PURE__*/_createClass(function FileLogger(params) {
  var _this = this;

  _classCallCheck(this, FileLogger);

  _defineProperty(this, "defaultFormat", function (json) {
    return JSON.stringify(json);
  });

  _defineProperty(this, "getCurrentDate", function () {
    if (_this.useUTC) {
      return _moment["default"].utc();
    } else {
      return (0, _moment["default"])();
    }
  });

  _defineProperty(this, "log", function (data) {
    try {
      var item = _this.format({
        timestamp: _this.getCurrentDate().format('YYYY-MM-DDTHH:mm:ssZ'),
        data: data
      });

      _this.stream.write(item + "\n");
    } catch (error) {
      console.log(data);
      console.log(error);
    }
  });

  _defineProperty(this, "getCurrentFile", function () {
    var filename = _this.filename.replace("%DATE%", _this.getCurrentDate().format(_this.logRotatePattern));

    return "".concat(_this.directory, "/").concat(filename);
  });

  this.format = params.format || this.defaultFormat;
  this.logRotatePattern = params.logRotatePattern || "YYYY-MM-DD";
  this.filename = params.filename;
  this.symLinkName = params.symLink;
  this.useUTC = params.useUTC;
  this.directory = params.directory;
  this.levels = params.levels || ['error', 'info', 'verbose']; // File rotation

  this.compressOnRotation = params.compressOnRotation;
  this.maxFileSizeMB = parseFloat(params.maxFileSizeMB || 20);
  this.maxRetainedFiles = parseFloat(params.maxRetainedFiles || 20);

  if (!_fs["default"].existsSync(this.directory)) {
    _fs["default"].mkdirSync(this.directory);
  }

  var streamOptions = {
    filename: "".concat(this.directory, "/").concat(this.filename),
    size: "".concat(this.maxFileSizeMB, "m"),
    frequency: "custom",
    end_stream: true,
    max_logs: this.maxRetainedFiles,
    date_format: this.logRotatePattern,
    utc: this.useUTC,
    create_symlink: !!this.symLinkName,
    symlink_name: this.symLinkName,
    verbose: false,
    audit_file: this.symLinkName ? "".concat(this.directory, "/.").concat(this.symLinkName.replace(".log", "")) : null
  };
  this.stream = _fileStreamRotator["default"].getStream(streamOptions);

  if (this.compressOnRotation) {
    this.stream.on('rotate', function (oldFile, newFile) {
      try {
        var tmpFile = newFile + ".tmp";

        var zip = _zlib["default"].createGzip();

        var read = _fs["default"].createReadStream(newFile);

        var write = _fs["default"].createWriteStream(tmpFile);

        read.pipe(zip).pipe(write);
        write.on('finish', function () {
          _fs["default"].unlinkSync(newFile);

          _fs["default"].renameSync(tmpFile, newFile);
        });
      } catch (error) {
        console.log(error); // Nothing else we can do
      }
    });
  }
});

exports["default"] = FileLogger;
;